import setuptools
 
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()    
 
setuptools.setup(name='ImportarSurvey', 
        version='1.4.0.0',
        author='Esri Colombia',
        author_email='mtorres@esri.co',
        license='MIT',
        long_description=long_description,
        long_description_content_type="text/markdown",
        url='https://dev.azure.com/esrico-con-uaecd-ladm/ImportarEncuestaActualizacionCatastral',
        description=('Módulo de Geoprocesamiento ArcGIS Pro para la Precarga e Inyeccion de Encuestas de Actualización Catastral'),
        python_requires= '>=3.7',
        classifiers=[
            'Programming Language :: Python :: 3.7',
            'License :: OSI Approved :: MIT License',
            'Operating System :: Microsoft :: Windows :: Windows 10',
            'Operating System :: MacOS',
            'Operating System :: Other OS',
            'Natural Language :: Spanish'
        ],
        package_dir={"": "src"},
        packages = setuptools.find_packages(where="src"),
        package_data={'ImportarSurvey':[
                    '*.json',
                    'esri/toolboxes/*',  
                    'esri/arcpy/*', 
                    'esri/help/gp/*',  
                    'esri/help/gp/toolboxes/*', 
                    'esri/help/gp/messages/*'] 
                    }, 
      )